@@ -1,28 +1,22 @@
 from __future__ import with_statement
 
 import re
-from flask import Blueprint
-from flask import render_template
-from flask_wtf.csrf import CsrfProtect
-from flask_wtf.csrf import validate_csrf, generate_csrf
-from .base import TestCase, MyForm, to_unicode
+import warnings
 
-csrf_token_input = re.compile(
-    r'name="csrf_token" type="hidden" value="([0-9a-z#A-Z-\.]*)"'
-)
+from flask import Blueprint, abort, render_template, request
+from flask import g
+from wtforms import ValidationError
 
-
-def get_csrf_token(data):
-    match = csrf_token_input.search(to_unicode(data))
-    assert match
-    return match.groups()[0]
+from flask_wtf._compat import FlaskWTFDeprecationWarning
+from flask_wtf.csrf import CSRFError, CSRFProtect, generate_csrf, validate_csrf
+from .base import MyForm, TestCase
 
 
 class TestCSRF(TestCase):
     def setUp(self):
         app = self.create_app()
         app.config['WTF_CSRF_SECRET_KEY'] = "a poorly kept secret."
-        csrf = CsrfProtect(app)
+        csrf = CSRFProtect(app)
         self.csrf = csrf
 
         @csrf.exempt
@@ -59,13 +53,13 @@ class TestCSRF(TestCase):
         response = self.client.post("/", data={"name": "danny"})
         assert response.status_code == 400
 
-        @self.csrf.error_handler
-        def invalid(reason):
-            return reason
+        @self.app.errorhandler(CSRFError)
+        def handle_csrf_error(e):
+            return e, 200
 
         response = self.client.post("/", data={"name": "danny"})
         assert response.status_code == 200
-        assert b'token missing' in response.data
+        assert b'The CSRF token is missing.' in response.data
 
     def test_invalid_csrf2(self):
         # tests with bad token
@@ -86,8 +80,9 @@ class TestCSRF(TestCase):
         assert response.status_code == 400
 
     def test_valid_csrf(self):
-        response = self.client.get("/")
-        csrf_token = get_csrf_token(response.data)
+        with self.client:
+            self.client.get('/')
+            csrf_token = g.csrf_token
 
         response = self.client.post("/", data={
             "name": "danny",
@@ -96,8 +91,9 @@ class TestCSRF(TestCase):
         assert b'DANNY' in response.data
 
     def test_prefixed_csrf(self):
-        response = self.client.get('/')
-        csrf_token = get_csrf_token(response.data)
+        with self.client:
+            self.client.get('/')
+            csrf_token = g.csrf_token
 
         response = self.client.post('/', data={
             'prefix-name': 'David',
@@ -106,8 +102,9 @@ class TestCSRF(TestCase):
         assert response.status_code == 200
 
     def test_invalid_secure_csrf(self):
-        response = self.client.get("/", base_url='https://localhost/')
-        csrf_token = get_csrf_token(response.data)
+        with self.client:
+            self.client.get('/', base_url='https://localhost/')
+            csrf_token = g.csrf_token
 
         response = self.client.post(
             "/",
@@ -116,7 +113,7 @@ class TestCSRF(TestCase):
             base_url='https://localhost/',
         )
         assert response.status_code == 400
-        assert b'failed' in response.data
+        assert b'The referrer header is missing.' in response.data
 
         response = self.client.post(
             "/",
@@ -130,7 +127,7 @@ class TestCSRF(TestCase):
             base_url='https://localhost/',
         )
         assert response.status_code == 400
-        assert b'not match' in response.data
+        assert b'The referrer does not match the host.' in response.data
 
         response = self.client.post(
             "/",
@@ -144,7 +141,7 @@ class TestCSRF(TestCase):
             base_url='https://localhost/',
         )
         assert response.status_code == 400
-        assert b'not match' in response.data
+        assert b'The referrer does not match the host.' in response.data
 
         response = self.client.post(
             "/",
@@ -158,11 +155,13 @@ class TestCSRF(TestCase):
             base_url='https://localhost/',
         )
         assert response.status_code == 400
-        assert b'not match' in response.data
+        assert b'The referrer does not match the host.' in response.data
 
     def test_valid_secure_csrf(self):
-        response = self.client.get("/", base_url='https://localhost/')
-        csrf_token = get_csrf_token(response.data)
+        with self.client:
+            self.client.get('/', base_url='https://localhost/')
+            csrf_token = g.csrf_token
+
         response = self.client.post(
             "/",
             data={"name": "danny"},
@@ -177,29 +176,20 @@ class TestCSRF(TestCase):
         assert response.status_code == 200
 
     def test_valid_csrf_method(self):
-        response = self.client.get("/")
-        csrf_token = get_csrf_token(response.data)
+        with self.client:
+            self.client.get('/')
+            csrf_token = g.csrf_token
 
         response = self.client.post("/csrf-protect-method", data={
             "csrf_token": csrf_token
         })
         assert response.status_code == 200
 
-    def test_invalid_csrf_method(self):
-        response = self.client.post("/csrf-protect-method", data={"name": "danny"})
-        assert response.status_code == 400
-
-        @self.csrf.error_handler
-        def invalid(reason):
-            return reason
-
-        response = self.client.post("/", data={"name": "danny"})
-        assert response.status_code == 200
-        assert b'token missing' in response.data
-
     def test_empty_csrf_headers(self):
-        response = self.client.get("/", base_url='https://localhost/')
-        csrf_token = get_csrf_token(response.data)
+        with self.client:
+            self.client.get('/', base_url='https://localhost/')
+            csrf_token = g.csrf_token
+
         self.app.config['WTF_CSRF_HEADERS'] = list()
         response = self.client.post(
             "/",
@@ -215,8 +205,10 @@ class TestCSRF(TestCase):
         assert response.status_code == 400
 
     def test_custom_csrf_headers(self):
-        response = self.client.get("/", base_url='https://localhost/')
-        csrf_token = get_csrf_token(response.data)
+        with self.client:
+            self.client.get('/', base_url='https://localhost/')
+            csrf_token = g.csrf_token
+
         self.app.config['WTF_CSRF_HEADERS'] = ['X-XSRF-TOKEN']
         response = self.client.post(
             "/",
@@ -239,9 +231,10 @@ class TestCSRF(TestCase):
         self.app.testing = True
         self.client.post("/", data={"name": "danny"})
 
-    def test_csrf_exempt(self):
-        response = self.client.get("/csrf-exempt")
-        csrf_token = get_csrf_token(response.data)
+    def test_csrf_exempt_view_with_form(self):
+        with self.client:
+            self.client.get('/', base_url='https://localhost/')
+            csrf_token = g.csrf_token
 
         response = self.client.post("/csrf-exempt", data={
             "name": "danny",
@@ -251,22 +244,23 @@ class TestCSRF(TestCase):
 
     def test_validate_csrf(self):
         with self.app.test_request_context():
-            assert not validate_csrf('ff##dd')
-            csrf_token = generate_csrf()
-            assert validate_csrf(csrf_token)
+            self.assertRaises(ValidationError, validate_csrf, None)
+            self.assertRaises(ValidationError, validate_csrf, 'invalid')
+            validate_csrf(generate_csrf())
 
     def test_validate_not_expiring_csrf(self):
         with self.app.test_request_context():
-            csrf_token = generate_csrf(time_limit=False)
-            assert validate_csrf(csrf_token, time_limit=False)
+            csrf_token = generate_csrf()
+            validate_csrf(csrf_token, time_limit=False)
 
     def test_csrf_token_helper(self):
         @self.app.route("/token")
         def withtoken():
             return render_template("csrf.html")
 
-        response = self.client.get('/token')
-        assert b'#' in response.data
+        with self.client:
+            response = self.client.get('/token')
+            assert re.search(br'token: ([0-9a-zA-Z\-._]+)', response.data)
 
     def test_csrf_blueprint(self):
         response = self.client.post('/bar/foo')
@@ -281,5 +275,47 @@ class TestCSRF(TestCase):
         def withtoken():
             return render_template("import_csrf.html")
 
-        response = self.client.get('/token')
-        assert b'#' in response.data
+        with self.client:
+            response = self.client.get('/token')
+            assert g.csrf_token in response.data.decode('utf8')
+
+    def test_csrf_custom_token_key(self):
+        with self.app.test_request_context():
+            # Generate a normal and a custom CSRF token
+            default_csrf_token = generate_csrf()
+            custom_csrf_token = generate_csrf(token_key='oauth_state')
+
+            # Verify they are different due to using different session keys
+            self.assertNotEqual(default_csrf_token, custom_csrf_token)
+
+            # However, the custom key can validate as well
+            validate_csrf(custom_csrf_token, token_key='oauth_state')
+
+    def test_old_error_handler(self):
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always', FlaskWTFDeprecationWarning)
+
+            @self.csrf.error_handler
+            def handle_csrf_error(reason):
+                return 'caught csrf return'
+
+            self.assertEqual(len(w), 1)
+            assert issubclass(w[0].category, FlaskWTFDeprecationWarning)
+            assert 'app.errorhandler(CSRFError)' in str(w[0].message)
+
+            rv = self.client.post('/', data={'name': 'david'})
+            assert b'caught csrf return' in rv.data
+
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always', FlaskWTFDeprecationWarning)
+
+            @self.csrf.error_handler
+            def handle_csrf_error(reason):
+                abort(401, 'caught csrf abort')
+
+            self.assertEqual(len(w), 1)
+            assert issubclass(w[0].category, FlaskWTFDeprecationWarning)
+            assert 'app.errorhandler(CSRFError)' in str(w[0].message)
+
+            rv = self.client.post('/', data={'name': 'david'})
+            assert b'caught csrf abort' in rv.data
