@@ -1,18 +1,9 @@
 from __future__ import with_statement
 
-import re
+from flask import g, json
 
-from .base import TestCase, MyForm, to_unicode
-
-csrf_token_input = re.compile(
-    r'name="csrf_token" type="hidden" value="([0-9a-z#A-Z-\.]*)"'
-)
-
-
-def get_csrf_token(data):
-    match = csrf_token_input.search(to_unicode(data))
-    assert match
-    return match.groups()[0]
+from flask_wtf.csrf import generate_csrf
+from .base import MyForm, TestCase, capture_logging, to_unicode
 
 
 class TestValidateOnSubmit(TestCase):
@@ -31,22 +22,24 @@ class TestValidateOnSubmit(TestCase):
         response = self.client.post("/", data={"name": "danny"})
         assert b'DANNY' in response.data
 
+    def test_json_data(self):
+        self.app.config['WTF_CSRF_ENABLED'] = False
+        response = self.client.post(
+            '/', content_type='application/json',
+            data=json.dumps({'name': 'Flask-WTF'})
+        )
+        assert b'FLASK-WTF' in response.data
+
 
 class TestValidateWithoutSubmit(TestCase):
-
     def test_unsubmitted_valid(self):
         class obj:
-            name = "foo"
+            name = 'foo'
 
         with self.app.test_request_context():
-            assert MyForm(obj=obj, csrf_enabled=False).validate()
-            fake_session = {}
-            t = MyForm(csrf_context=fake_session).generate_csrf_token(
-                fake_session
-            )
-            assert MyForm(
-                obj=obj, csrf_token=t,
-                csrf_context=fake_session).validate()
+            assert MyForm(obj=obj, meta={'csrf': False}).validate()
+            t = generate_csrf()
+            assert MyForm(obj=obj, csrf_token=t).validate()
 
 
 class TestHiddenTag(TestCase):
@@ -63,17 +56,19 @@ class TestCSRF(TestCase):
     def test_csrf_token(self):
 
         response = self.client.get("/")
-        snippet = (
-            '<div style="display:none;">'
-            '<input id="csrf_token" name="csrf_token" type="hidden" value'
-        )
+        snippet = '<input id="csrf_token" name="csrf_token" type="hidden" value'
         assert snippet in to_unicode(response.data)
 
     def test_invalid_csrf(self):
+        from flask_wtf.csrf import logger
+
+        with capture_logging(logger) as handler:
+            response = self.client.post("/", data={"name": "danny"})
 
-        response = self.client.post("/", data={"name": "danny"})
         assert b'DANNY' not in response.data
-        assert b'CSRF token missing' in response.data
+        assert b'The CSRF token is missing.' in response.data
+        self.assertEqual(1, len(handler))
+        self.assertEqual('The CSRF token is missing.', handler[0].message)
 
     def test_csrf_disabled(self):
 
@@ -96,27 +91,23 @@ class TestCSRF(TestCase):
         assert response.status_code == 200
 
     def test_valid_csrf(self):
+        with self.client:
+            self.client.get('/')
+            csrf_token = g.csrf_token
 
-        response = self.client.get("/")
-        csrf_token = get_csrf_token(response.data)
-
-        response = self.client.post("/", data={"name": "danny",
-                                               "csrf_token": csrf_token})
+        response = self.client.post('/', data={
+            'name': 'danny',
+            'csrf_token': csrf_token
+        })
         assert b'DANNY' in response.data
 
     def test_double_csrf(self):
-
-        response = self.client.get("/")
-        csrf_token = get_csrf_token(response.data)
+        with self.client:
+            self.client.get('/')
+            csrf_token = g.csrf_token
 
         response = self.client.post("/two_forms/", data={
             "name": "danny",
             "csrf_token": csrf_token
         })
         assert response.data == b'OK'
-
-    def test_valid_csrf_data(self):
-        with self.app.test_request_context():
-            form = MyForm()
-            csrf_token = get_csrf_token(form.csrf_token())
-            assert form.validate_csrf_data(csrf_token)
